﻿using System;
using System.Collections.Generic;

using Codice.CmdRunner;

namespace MergeWithCodeReviewTrigger
{
    class Program
    {
        static int Main(string[] args)
        {
            string mergeSource = Environment.GetEnvironmentVariable("PLASTIC_MERGE_SOURCE");

            string[] mergeSourceFields = mergeSource.Split('@');
            string csetId = mergeSourceFields[0].Substring(mergeSourceFields[0].IndexOf(':') + 1);
            string fullBranchName = mergeSourceFields[1].Substring(mergeSourceFields[1].IndexOf(':') + 1);
            string repSpec = mergeSourceFields[2] + "@" + mergeSourceFields[3];
            string shortBranchName = fullBranchName.Substring(fullBranchName.LastIndexOf('/') + 1);

            string branchId = GetBranchIdFromName(fullBranchName, shortBranchName, repSpec);

            if (HasBranchCodeReviewApproved(branchId, repSpec))
                return 0;
            else
            {
                Console.WriteLine(string.Format("Approved code review not found for branch:{0} (branchid:{1})", fullBranchName, branchId));
                return 1;
            }
            
        }

        private static bool HasBranchCodeReviewApproved(string branchId, string repoSpec)
        {
            string command = string.Format("cm find \"review where targetid = '{0}' and status = '1'\" --format=\"OK\" --nototal on repository '{1}'", branchId, repoSpec);
            string cmdout = CmdRunner.ExecuteCommandWithStringResult(command, Environment.CurrentDirectory);

            List<string> resultList = StringHelper.GetListStringResults(cmdout);
            foreach (string cmdoutline in resultList)
            {
                if (cmdoutline.StartsWith("OK"))
                    return true;
            }

            return false;
        }

        private static string GetBranchIdFromName(string fullBranchName, string shortBranchName, string repoSpec)
        {
            string command = string.Format("cm find \"branch where name = '{0}'\" --format={{id}}:{{name}} --nototal on repository '{1}'", shortBranchName, repoSpec);
            string cmdout = CmdRunner.ExecuteCommandWithStringResult(command, Environment.CurrentDirectory);

            List<string> resultList = StringHelper.GetListStringResults(cmdout);
            foreach (string cmdoutline in resultList)
            {
                string[] lineFileds = StringHelper.Clean(cmdoutline).Split(':');

                if (lineFileds[1] == fullBranchName)
                    return lineFileds[0];
            }
           
            return string.Empty;
        }
    }
}
